package ip1.ex14;

import java.io.File;
import java.net.URL;
import java.util.Vector;


/** A file or directory entry.
 *
 *  @version $Id: JFtpEntry.java,v 1.7 2003/10/16 16:31:55 sharky Exp $
 */
public class JFtpEntry
{
  public final static int REMOTE = 10;
  public final static int LOCAL  = 20;
  public final static int FILE   = 30;
  public final static int DIR    = 40;
  public final static int BAD    = 50;
  public String name;
  public int location;
  public int type;
  public URL url;
  public long size = 0;
  private static long kB = 1024;
  private static long MB = kB * kB;
  private static long GB = MB * kB;

  /** The constructor. */
  public JFtpEntry(String name, int location, int type, URL url)
  {
    this.name = name;
    this.location = location;
    this.type = type;
    this.url = url;
  }

  /** Constructor for local entries. */
  public JFtpEntry(File file)
  {
    this.name = file.getName();
    this.location = LOCAL;
    if (file.isDirectory())
      this.type = DIR;
    else
      this.type = FILE;
    this.size = file.length();
    this.url = null;
  }

  /** Constuctor for remote entries. */
  public JFtpEntry(String toParse, URL url)
  {
    this.url = url;
    this.location = REMOTE;

    String bits[] = toParse.split("\\s+");

    // we must have at least 8 fields
    if (bits.length >= 9)
    {
      // type
      if (bits[0].charAt(0) == 'd')
        type = DIR;
      else
        type = FILE;

      // size
      size = Integer.parseInt(bits[4]);

      // name
      name = bits[8];
      for (int j = 9; j < bits.length; j++)
        name += " " + bits[j];

    } else if (bits.length >= 4) {
      // type
      if (bits[2].equals("<DIR>"))
        type = DIR;
      else
        type = FILE;

      // name
      name = bits[3];
      for (int j = 4; j < bits.length; j++)
        name += " " + bits[j];

    } else {
      type = BAD;
      name = "<badfile>";
      System.out.println("could not parse : " + toParse);
    }
  }


  public static String formatSize(long theSize)
  {
    if (theSize < kB)
      return "" + theSize + " B";
    else if (theSize < MB)
      return "" + (((float) ( theSize*10/kB) ) / 10) + " kB";
    else if (theSize < GB)
      return "" + (((float) ( theSize*10/MB) ) / 10) + " MB";
    else
      return "" + (((float) ( theSize*10/GB) ) / 10) + " MB";
  }

  /** return a vector containing a local directory listing. */
  public static Vector makeLocalList(File localDir)
  {
    String lines[] = localDir.list();
    Vector entries = new Vector();

    // add entries to the list
    entries.addElement(new JFtpEntry(new File(localDir,".")));
    entries.addElement(new JFtpEntry(new File(localDir,"..")));
    for (int i = 0; i < lines.length; i++) {
      JFtpEntry entry = new JFtpEntry(new File(localDir,lines[i]));
      if ( !entry.name.equals(".") &&
           !entry.name.equals("..") )
        entries.addElement(entry);
    }

    return orderList(entries);
  }


  /** return a vector containing a local directory listing. */
  public static Vector makeRemoteList(URL remoteUrl, String[] lines)
  {
    Vector entries = new Vector();

    // add entries to the list
    entries.addElement(new JFtpEntry(".",REMOTE, DIR, remoteUrl));
    entries.addElement(new JFtpEntry("..",REMOTE, DIR, remoteUrl));
    for (int i = 0; i < lines.length; i++) {
      JFtpEntry entry = new JFtpEntry(lines[i],remoteUrl);
      if ( !entry.name.equals(".") &&
           !entry.name.equals("..") )
        entries.addElement(entry);
    }

    return orderList(entries);
  }


  /** orders the elements of a directory listing : directories first, then files. */
  public static Vector orderList(Vector entries)
  {
    Vector newEntries = new Vector();

     // add directories first
     for (int i = 0; i < entries.size(); i++) {
       JFtpEntry entry = (JFtpEntry) entries.elementAt(i);
       if (entry.type == DIR) newEntries.add(entry);
     }

     // then add files
     for (int i = 0; i < entries.size(); i++) {
       JFtpEntry entry = (JFtpEntry) entries.elementAt(i);
       if (entry.type != DIR) newEntries.add(entry);
     }
     return newEntries;
  }


  /** Return string representation. */
  public String toString()
  {
    String out = "[";
    switch (type) {
    case FILE:
      out += '-';
      break;
    case DIR:
      out += 'd';
      break;
    }
    out += "] " + name;
    if ((type == FILE) && (size!=0)) {
      out += "  ( " + formatSize(size) + ") ";
    }
    return out;
  }
}
