package ip1.ex14;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.net.*;


/** The JFtpWindow implements JFtp's GUI.
 *
 * @version $Id: JFtpWindow.java,v 1.15 2003/10/20 00:37:26 sharky Exp $
 */
/** The JFtp main window. */
public class JFtpWindow extends JFrame implements ActionListener,ListSelectionListener,WindowListener,ChangeListener
{
  /** The main thread. */
  JFtp jftp;

// GUI
  /** The field where the user enters the URL to browse. */
  protected JTextField urlField;
  /** The scrollable text area where we display the log. */
  protected JTextArea logArea;
  /** The local file list. */
  protected JList localList;
  /** The remote file list. */
  protected JList remoteList;

  /** The Connect button. */
  private JButton btnConnect;
  /** The Disconnect button. */
  private JButton btnDisconnect;
  /** The Download button. */
  private JButton btnDownload;
  /** The Quit button. */
  private JButton btnQuit;
  /** The Upload button. */
  private JButton btnUpload;
  /** The local Delete button. */
  private JButton btnLocalDelete;
  /** The local Refresh button. */
  private JButton btnLocalRefresh;
  /** The remote Delete button. */
  private JButton btnRemoteDelete;
  /** The remote Refresh button. */
  private JButton btnRemoteRefresh;

  /** Spinner for the number of threads. */
  private JSpinner spinnerThreads;
  /** Passive mode checkbox. */
  private JCheckBox checkPassive;

  /** The constructor.
   *
   * @param jftp the JFtp thread we send events to
   */
  public JFtpWindow(JFtp jftp)
  {
    super("JFtp " +jftp.version);
    this.jftp = jftp;
    addWindowListener(this);

    // set up the url text field
    JPanel urlPanel = new JPanel();
    urlPanel.setLayout(new BoxLayout(urlPanel,BoxLayout.X_AXIS));
    urlPanel.add(Box.createHorizontalStrut(10));
    JLabel urlLabel = new JLabel("url");
    urlPanel.add(urlLabel);
    urlPanel.add(Box.createHorizontalStrut(10));
    urlField = new JTextField(jftp.argUrl);
    urlField.addActionListener(this);
    urlPanel.add(urlField);
    urlPanel.add(Box.createHorizontalGlue());
    btnConnect = new JButton("Connect");
    btnConnect.addActionListener(this);
    urlPanel.add(btnConnect);
    urlPanel.setMaximumSize(new Dimension(2000,50));

    // the local file list
    localList = new JList();
    localList.addListSelectionListener(this);
    JScrollPane localScroll = new JScrollPane(localList);
    //localScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    // the local buttons
    JPanel localBtnPanel = new JPanel();
    localBtnPanel.setLayout(new GridLayout(1,0));
    btnLocalRefresh = new JButton("Refresh");
    btnLocalRefresh.addActionListener(this);
    localBtnPanel.add(btnLocalRefresh);
    btnLocalDelete = new JButton("Delete");
    btnLocalDelete.addActionListener(this);
    localBtnPanel.add(btnLocalDelete);
    localBtnPanel.setMaximumSize(new Dimension(1000,50));

    // the whole local panel
    JPanel localPanel = new JPanel();
    localPanel.setLayout(new BoxLayout(localPanel,BoxLayout.Y_AXIS));
    localPanel.add(localScroll);
    localPanel.add(localBtnPanel);

    // the action buttons
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout());
    JPanel btnSubPanel = new JPanel();
    btnSubPanel.setLayout(new GridLayout(0,1));

    btnUpload = new JButton("Upload -->");
    btnUpload.addActionListener(this);
    btnSubPanel.add(btnUpload);

    btnDownload = new JButton("<-- Download");
    btnDownload.addActionListener(this);
    btnSubPanel.add(btnDownload);

    btnDisconnect = new JButton("Disconnect");
    btnDisconnect.addActionListener(this);
    btnSubPanel.add(btnDisconnect);

    btnQuit = new JButton("Quit");
    btnQuit.addActionListener(this);
    btnSubPanel.add(btnQuit);

    // do we use passive mode?
    checkPassive = new JCheckBox("Passive mode",jftp.argPassive);
    checkPassive.addActionListener(this);
    btnSubPanel.add(checkPassive);

    // number of concurrent threads
    JPanel threadsPanel = new JPanel();
    threadsPanel.setLayout(new BoxLayout(threadsPanel,BoxLayout.X_AXIS));
    threadsPanel.add(new JLabel("Maximum theads "));
    spinnerThreads = new JSpinner(new SpinnerNumberModel(jftp.argMaxThreads, 1, 20, 1));
    spinnerThreads.addChangeListener(this);

    threadsPanel.add(spinnerThreads);
    btnSubPanel.add(threadsPanel);

    btnSubPanel.setPreferredSize(new Dimension(160,150));
    buttonPanel.add(btnSubPanel);

    // the remote file list
    remoteList = new JList();
    remoteList.addListSelectionListener(this);
    JScrollPane remoteScroll = new JScrollPane(remoteList);
    //remoteScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    // the remote buttons
    JPanel remoteBtnPanel = new JPanel();
    remoteBtnPanel.setLayout(new GridLayout(1,0));
    btnRemoteRefresh = new JButton("Refresh");
    btnRemoteRefresh.addActionListener(this);
    remoteBtnPanel.add(btnRemoteRefresh);
    btnRemoteDelete = new JButton("Delete");
    btnRemoteDelete.addActionListener(this);
    remoteBtnPanel.add(btnRemoteDelete);
    remoteBtnPanel.setMaximumSize(new Dimension(1000,50));
    // the whole remote panel
    JPanel remotePanel = new JPanel();
    remotePanel.setLayout(new BoxLayout(remotePanel,BoxLayout.Y_AXIS));
    remotePanel.add(remoteScroll);
    remotePanel.add(remoteBtnPanel);

    // the navigator tab
    JPanel splitPanel = new JPanel();
    splitPanel.setLayout(new BoxLayout(splitPanel,BoxLayout.X_AXIS));
    splitPanel.add(localPanel);
    splitPanel.add(buttonPanel);
    splitPanel.add(remotePanel);

    // scrollable text area for the log
    logArea = new JTextArea();
    JScrollPane logScroll = new JScrollPane(logArea);
    logScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

    // create tabbed pane
    JTabbedPane tabbedPane = new JTabbedPane();
    tabbedPane.add("file navigator",splitPanel);
    tabbedPane.add("debug output",logScroll);
    tabbedPane.setPreferredSize(new Dimension(700,300));

    // thread list
    JScrollPane threadScroll = new JScrollPane(jftp.threadTree);
    threadScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    threadScroll.setPreferredSize(new Dimension(700,150));


    // put it all together
    Container contentPane = getContentPane();
    contentPane.setLayout(new BoxLayout(contentPane,BoxLayout.Y_AXIS));
    contentPane.add(urlPanel);
    contentPane.add(Box.createVerticalStrut(10));
    contentPane.add(tabbedPane);
    contentPane.add(threadScroll);

    pack();
    setVisible(true);
  }



  /** Gets the selected files.
   *
   * @param list the list we read from
   */
  public Vector getSelectedFiles(JList list)
  {
    Vector items = new Vector();
    Object entries[] = list.getSelectedValues();
    for (int i = 0; i < entries.length; i++) {
      JFtpEntry entry = (JFtpEntry) entries[i];
      // do we have an entry which is a file ?
      if ((entry != null) && (entry.type != JFtpEntry.DIR))
      {
	// set the 'URL' for local files so we know where to upload to
        if ((entry.location == JFtpEntry.LOCAL) && (jftp.getControl() != null))
          entry.url = jftp.getControl().remoteUrl;
        items.addElement(entry);
      }
    }

    return items;
  }


  /** This gets called when the user presses return in the url field or presses
   *  one of the window's buttons.
   *
   * @param evt
   *
   * @see JFtp#argPassive
   * @see JFtp#delete
   * @see JFtp#queueDownload
   * @see JFtp#queueUpload
   * @see JFtp#refreshLocalList
   * @see JFtp#refreshRemoteList
   * @see JFtp#shutdown
   * @see JFtpControl#queueShutdown
   */
  public void actionPerformed(ActionEvent evt)
  {

    if ( (evt.getSource() == urlField)
      || (evt.getSource() == btnConnect) ) {

  // connect, right now
      try {
        jftp.connect(new URL(urlField.getText()));
      } catch (MalformedURLException e) {}

    } else if (evt.getSource() == btnDisconnect) {

  // disconnect, right now
      jftp.reset();

    } else if (evt.getSource() == btnQuit) {

  // quit, right now
      jftp.shutdown();


    } else if (evt.getSource() == btnLocalDelete) {

  // delete local files, right now
      Vector entries = getSelectedFiles(localList);
      for (int i = 0; i < entries.size(); i++)
        jftp.delete((JFtpEntry) entries.elementAt(i));
  // refresh local file list
      jftp.refreshLocalList();

    } else if (evt.getSource() == btnLocalRefresh) {

  // refresh local file list, right now
      jftp.refreshLocalList();

    } else if (evt.getSource() == btnRemoteDelete) {

  // delete remote files
      Vector entries = getSelectedFiles(remoteList);
      for (int i = 0; i < entries.size(); i++)
        jftp.delete((JFtpEntry) entries.elementAt(i));
      jftp.refreshRemoteList();

    } else if (evt.getSource() == btnRemoteRefresh) {

  // refresh remote file list
      jftp.refreshRemoteList();

    } else if (evt.getSource() == btnDownload) {

  // download file(s)
      Vector entries = getSelectedFiles(remoteList);
      for (int i = 0; i < entries.size(); i++)
        jftp.queueDownload((JFtpEntry) entries.elementAt(i));

    } else if (evt.getSource() == btnUpload) {

  // upload file(s)
      Vector entries = getSelectedFiles(localList);
      for (int i = 0; i < entries.size(); i++)
        jftp.queueUpload((JFtpEntry) entries.elementAt(i));

    } else if (evt.getSource() == checkPassive) {

  // set passive mode
      jftp.setPassive(checkPassive.isSelected());
    }

  }


  /** This gets called when the user changes the value of the JSpinner
   *  which controls the maximum number of consecutive control threads.
   *
   * @see JFtp#argMaxThreads
   */
  public void stateChanged(ChangeEvent e) {
    if (e.getSource() == spinnerThreads) {
      jftp.argMaxThreads = ((SpinnerNumberModel)spinnerThreads.getModel()).getNumber().intValue();
    }
  }


  /** This gets called when a user changes selection in the list. */
  public void valueChanged(ListSelectionEvent e)
  {
    if (e.getValueIsAdjusting())
      return;


    if (e.getSource() == localList) {
    // catch clicks on local directories
      if (localList.getSelectedValue() == null)
        return;

      JFtpEntry entry = (JFtpEntry) localList.getSelectedValue();
      if (entry.type != JFtpEntry.DIR)
        return;

      jftp.localCwd(entry.name);
    } else if (e.getSource() == remoteList) {
    // catch clicks on remote directories
      if (remoteList.getSelectedValue() == null)
        return;

      JFtpEntry entry = (JFtpEntry) remoteList.getSelectedValue();
      if (entry.type != JFtpEntry.DIR)
        return;

      if (jftp.getControl() != null) {
        jftp.getControl().queueCwd(entry);
        jftp.getControl().queueList();
      }
    }
  }


  /** Catch the closing of the window. */
  public void windowClosing(WindowEvent e) {
    jftp.shutdown();
  }

  public void windowClosed(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}

}

